@@ -14,7 +14,7 @@ class Sync(Runner):
         self.queue = Queue()
 
     def kill(self, task_id):
-        self.executor.kill(task_id)
+        return self.executor.kill(task_id)
 
     def reconcile(self, task_config):
         self.executor.reconcile(task_config)
